/* main.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "gtkProgram.h"
#include "paloma.h"
#include "database.h"
#include "postgresDatabase.h"
#include "dataCell.h"
#include "tableInfo.h"
#include "sfClient.h"
#include "scan_directory.h"
#include "create_tables.h"

#include <gtk--.h>
#include <vector>
#include <time.h>
#include <stdlib.h>

class PalomaMain : public GtkProgram {
public:
  PalomaMain();

  virtual void init();
  virtual void run();

protected:
  virtual void handle_opts(const string &opt, const string &arg);

  virtual void help();
  virtual void describe_options();

  bool _show_dove;
  int _sfserver_port;
  string _sfserver_host;
  vector<string> _cds;
};

PalomaMain::
PalomaMain() : GtkProgram("paloma") {
#ifdef HAVE_CDAUDIO
  define_opt("cd", true);
#endif
  define_opt("nodove", false);
  define_opt("sfhost", true);
  define_opt("sfport", true);
  _show_dove = true;
}

void PalomaMain::
init() {
  srand(time(NULL));
  GtkProgram::init();
}

void PalomaMain::
run() {
  // First, create any tables that are missing.  Normally, this will
  // only do anything the first time Paloma is run, in which case it
  // will create all the necessary tables.
  create_tables();

  // Although it isn't strictly necessary to call define_all_tables(),
  // since the TableDef class can look up each table on demand, we do
  // it anyway to make available the list of all fields in the world,
  // so that Playlists can look them up and format their columns
  // properly.
  define_all_tables();

  Dataset data;
  data[TI_configure] = TableDataProxy(TI_configure, 1);

  // Which CD devices do we have?
  if (_cds.empty()) {
    scan_directory("/dev/cdrom*", _cds);
  }

  // This fills in the global Paloma variable.  It must be a global
  // variable so other windows can access the toplevel window, for
  // instance, to get the dove bitmap.
  paloma = new Paloma(data, _dbname, _cds, _show_dove);

  GtkProgram::run();

  delete paloma;
}

void PalomaMain::
handle_opts(const string &opt, const string &arg) {
  if (opt == "cd") {
    _cds.push_back(arg);

  } else if (opt == "nodove") {
    _show_dove = false;

  } else if (opt == "sfport") {
    SFClient::_port = atoi(arg.c_str());

  } else if (opt == "sfhost") {
    SFClient::_hostname = arg;

  } else {    
    GtkProgram::handle_opts(opt, arg);
  }
}

void PalomaMain::
help() {
  cerr <<
    "\n"
    "Paloma is a music database management program.  It's a graphical program to\n"
    "assist you in organizing, retrieving, and playing collections of songs stored\n"
    "on your computer's hard disk.\n";

  GtkProgram::help();
}

void PalomaMain::
describe_options() {
  cerr <<
#ifdef HAVE_CDAUDIO
    "  -cd dev    Specify a CD-ROM device that may be used to scan CD's.  This\n"
    "             parameter may be given multiple times if your machine has\n"
    "             multiple CD-ROM devices.  If it is omitted, the default is all\n"
    "             of the devices matching /dev/cdrom*.\n"
#endif
    "  -nodove    Don't show the dove picture in the main window.  You might want\n"
    "             to eliminate this if you have a very slow connection to your X\n"
    "             server, for instance, or if you just dislike doves.\n"
    "  -sfhost h  Attempt to contact the sfserver on the host named 'h', instead \n"
    "             of the local host.  This is strictly for playing sound files,\n"
    "             and is not related to the database host.\n"
    "  -sfport p  Contact the sfserver on the indicated port, instead of the\n"
    "             default sfserver port, " << SFClient::_port << ".\n";
  GtkProgram::describe_options();
}

int
main(int argc, char *argv[]) {
  PalomaMain m;
  m.command_line(argc, argv);
  m.init();
  m.run();
  return 0;
}
