/* mySQLDatabase.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef MYSQLDATABASE_H
#define MYSQLDATABASE_H

#ifdef USE_MYSQL

#include "database.h"
#include "dataCell.h"

#include <mysql.h>

class MySQLDatabase : public Database {
public:
  MySQLDatabase(const string &name, const string &host, int port,
		const string &user, const string &passwd);
  virtual ~MySQLDatabase();

  virtual QueryResult sql_command(string command,
				  bool return_failures = false);

  virtual bool begin();
  virtual bool commit();
  virtual bool rollback();

  virtual bool has_sequence(const string &name);
  virtual bool create_sequence(const string &name);
  virtual int get_next_sequence(const string &name);

  virtual string concat(const string &a, const string &b);

  virtual bool has_table(const string &name);

  void check_db();
  DataCell::Type get_type(int type_id);

  static void extra_args(const string &args);

private:
  typedef map<int, DataCell::Type> Types;
  Types _types;
  
  MYSQL *_connection;

  static string _socket;
};

#endif  // USE_MYSQL

#endif
